
VERSION = '0.9.4'

USER_AGENT_STRING = 'pyShortUrl v%s' % VERSION

SUPPORTED_SERVICES = (
    'goo.gl',
    'bit.ly',
    'git.io',
    'tinyurl.com',
    'v.gd',
    'is.gd',
)

SUPPORTED_DOMAINS = (
    'bit.ly',
    'bitly.com',
    'goo.gl',
    'git.io',
    'is.gd',
    'j.mp',
    'tinyurl.com',
    'v.gd',
)
